﻿using System;
using System.IO;
using System.Threading;
using System.Windows.Forms;
using System.Diagnostics;
using System.Text;

namespace Alden_USB_CTR_15
{
    public partial class Form1 : Form
    {
        private System.Drawing.Image dotGreen, dotDark;
        public TextBox[] CountsDisplay;
        public PictureBox[] Counting;
        public uint seconds;
        public Stream logStream;
        private DateTime logStartTime, logEndTime;
        private bool nonNumberEntered = false;
        private static Thread logThread = null;
        private double secondsPerStatusUpdate = 0.1;
        private UInt16[] oldCounts;
        private int[] CountingTimeout = new int[15];
        bool logThreadDone = false;
        private UInt16[] endCounts = new UInt16[15];
        private uint RunNumber = 0;

        public Form1()
        {
            InitializeComponent();
            CountsDisplay = new TextBox[15];
            Counting = new PictureBox[15];
        }

        private void doLogThread()
        {
            logStartTime = DateTime.Now;
            long beginTime = Stopwatch.GetTimestamp();
            long endTime = beginTime + seconds * Stopwatch.Frequency;
            long ticksPerStatusUpdate = Convert.ToInt64(Stopwatch.Frequency * secondsPerStatusUpdate);
            long nextStatusUpdateTime = beginTime + ticksPerStatusUpdate;

            do
            {
                long now = Stopwatch.GetTimestamp();
                if ( now >= nextStatusUpdateTime )
                {
                    nextStatusUpdateTime += ticksPerStatusUpdate;
                    Invoke((MethodInvoker)delegate
                    {
                        displayLogStatus( (now - beginTime) / (endTime-beginTime) );
                    });
                }

                if (now >= endTime)
                    logThreadDone = true;

            } while (!logThreadDone);

            for (uint i = 0; i < 15; i++)
            {
                byte nullcount = 0xFF;
                UInt16 Count = 0;
                var status = AIOUSBNet.AIOUSB.CTR_8254ReadStatus(AIOUSBNet.AIOUSB.diOnly, 0, i, ref Count, ref nullcount);
                if (status != 0) LogText.Text += "thread status: " + status.ToString() + "\n";

                if ((nullcount & 0b01000000) == 0)
                    endCounts[i] = Convert.ToUInt16(65536-Count);
                else
                    endCounts[i] = 0;
            }
            logEndTime = DateTime.Now;

            Invoke((MethodInvoker)delegate {
                closeLog();
            });

         
        }

        private void closeLog()
        {
            // save endCounts to CSV, along with note for log row.text
            String LogLine = DateTime.Now.ToString("HH:mm:ss") + "," + RunNumber.ToString() + "," + seconds.ToString();
            for (int i = 0; i < 15; i++)
                LogLine += "," + endCounts[i].ToString()
            ;
            LogLine += ",\"" + NoteTextBox.Text + "\"";
            LogLine += "\r\n";
            byte[] LogBytes = new UTF8Encoding(true).GetBytes(LogLine);

            logStream = File.Open(logPath.Text, FileMode.Append, FileAccess.Write, FileShare.Read);
            logStream.Write(LogBytes, 0, LogBytes.Length);
            logStream.Close();

            TimingAccuracyLabel.Text = "Actual Duration: " + (logEndTime - logStartTime).ToString("g");

            progressBar.Visible = false;
            timer1.Enabled = true;
            buttonLog.Enabled = true;
        }

        private void displayLogStatus(double progress)
        {
            progressBar.Value = Convert.ToInt32(progress * progressBar.Maximum);
            Tick();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            init();
        }

        private void c11_CheckedChanged(object sender, EventArgs e)
        {
            (sender as CheckBox).Text = (sender as CheckBox).Checked ? "Counting" : "";
        }
        private void init()
        {
            var status = AIOUSBNet.AIOUSB.ResolveDeviceIndex(AIOUSBNet.AIOUSB.diOnly);
            if (status != 0) LogText.Text += "init status: " + status.ToString() + "\n";

            // J2H TODO Device Detection (support 2 cards!!) (EEPROMByte!)

            timer1.Enabled = false;
            dotGreen = c00.Image;
            dotDark = c01.Image;

            progressBar.Minimum = 0;
            progressBar.Maximum = 1000;

            for (uint i = 0; i < 15; i++)
            {
                Control[] edits = this.Controls.Find("textBox" + (i + 1).ToString(), true);
                edits[0].Text += " foo " + i.ToString();
                CountsDisplay[i] = edits[0] as TextBox;
                CountsDisplay[i].Text = "0";

                Control[] checks = this.Controls.Find("c" + i.ToString().PadLeft(2, '0'), true);
                Counting[i] = checks[0] as PictureBox;
                Counting[i].Image = dotDark;
                Counting[i].Tag = 0;

                CountingTimeout[i] = 0;
            }
            timer1.Enabled = true;

            { // J2H plausibly improvable
                logPath.Text = Environment.CurrentDirectory;
                if (logPath.Text[logPath.TextLength - 1] != '\\')
                    logPath.Text += '\\';
                logPath.Text += DateTime.Now.ToString("yyyyMMdd HHmmss") + " log.csv";
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            Tick();
        }

        private void Tick()
        {
            UInt16[] counts = new UInt16[CountsDisplay.Length];
            var status = AIOUSBNet.AIOUSB.CTR_8254ReadAll(AIOUSBNet.AIOUSB.diOnly, counts);
            if (status != 0) LogText.Text += "Tick status: " + status.ToString() + "\n";

            if (oldCounts == null)
            {
                oldCounts = counts;
                return;
            }

            for (uint i = 0; i < 15; i++)
            {
                if (oldCounts[i] != counts[i])
                {
                    Counting[i].Image = dotGreen;
                    CountsDisplay[i].Text = counts[i].ToString();
                    CountingTimeout[i] = 10;
                }
                else if (CountingTimeout[i] < 0) { }
                else if (CountingTimeout[i] == 0)
                {
                    Counting[i].Image = dotDark;
                    CountingTimeout[i] = -1;
                }
                else
                {
                    CountingTimeout[i]--;
                }
            }
            oldCounts = counts;
        }

        private void textBox17_KeyDown(object sender, KeyEventArgs e)
        {
            nonNumberEntered = false;
            if (e.KeyCode < Keys.D0 || e.KeyCode > Keys.D9)
                if (e.KeyCode < Keys.NumPad0 || e.KeyCode > Keys.NumPad9)
                    if (e.KeyCode != Keys.Back)
                        nonNumberEntered = true;

            if (Control.ModifierKeys == Keys.Shift)
                nonNumberEntered = true;
        }

        private void textBox17_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (nonNumberEntered == true)
                e.Handled = true;
        }

        private void buttonLog_Click(object sender, EventArgs e)
        {
            logThreadDone = false;
            buttonLog.Enabled = false;
            uint.TryParse(logSeconds.Text, out seconds);
            RunNumber++;
            RunNumberLabel.Text = "Run: " + RunNumber.ToString();
            progressBar.Visible = true;

            timer1.Enabled = false;
            oldCounts = null;
            for (uint i = 0; i < 15; i++)
            {
                var status = AIOUSBNet.AIOUSB.CTR_8254ModeLoad(AIOUSBNet.AIOUSB.diOnly, 0, i, 2, 0);
                if (status != 0) LogText.Text += "buttonLog status: " + status.ToString() + "\n";
            }

            logThread = new Thread(new ThreadStart(doLogThread));
            logThread.Priority = ThreadPriority.Highest; // J2H TODO: make config option
            logThread.Start();
        }

        private void logSeconds_TextChanged(object sender, EventArgs e)
        {
            
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            logThreadDone = true;
            if (logThread != null)
                while (logThread.IsAlive)
                    Application.DoEvents();
            
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void LogFileBrowse(object sender, EventArgs e)
        {
            saveFileDialog1.FileName = Path.GetFileName(logPath.Text);
            saveFileDialog1.InitialDirectory = Path.GetDirectoryName(logPath.Text);
            saveFileDialog1.Filter = "csv files (*.csv)|*.csv";
            saveFileDialog1.FilterIndex = 1;
            saveFileDialog1.RestoreDirectory = true;
            if ( saveFileDialog1.ShowDialog() == DialogResult.OK )
            {
                logPath.Text = saveFileDialog1.FileName;
            }

        }
    }
}
